﻿Module mdlGraphics
    Public NeedRefresh As Boolean
    Public NeedRedraw As Boolean
    Public ObjectHeight(MenuLoc.EndMenu) As Integer
    Public BackBufferBitMap As Bitmap
    Public BackBuffer As System.Drawing.Graphics
    Public InnerDrawSize As Size
    Public MenuDrawSize As Size

    Public ButtonDrawingRightSize As Size
    Public ButtonDrawingBottomSize As Size
    Public ButtonRightItemHeight As Integer
    Public ButtonBottomItemWidth As Integer
    Public GraphDrawingSize As Size

    Public IconSize As Size

    Public BackBufferDrawBitMap As Bitmap
    Public BackBufferDraw As System.Drawing.Graphics

    Public BackBufferMenuBitMap As Bitmap
    Public BackBufferMenu As System.Drawing.Graphics

    Public MenuBitMap As Bitmap
    Public MenuDraw As System.Drawing.Graphics

    Public MenuIconBitMap(MenuLoc.EndMenu) As Bitmap
    Public MenuIcon As System.Drawing.Graphics

    Public BufferBitMap(MenuLoc.EndMenu) As Bitmap
    Public Buffer As System.Drawing.Graphics

    Public EllapsedTime As Single
    Public FrapsCount As Single
    Public FPS As Single

    Public ForegroundColor As Color = Color.White
    Public BackgroundColor As Color = Color.Black

    Public BlackDrawingPen As New Pen(ForegroundColor, 1)
    Public BlackDrawingBrush As SolidBrush = New SolidBrush(ForegroundColor)
    Public WhiteDrawingPen As New Pen(BackgroundColor, 1)
    Public WhiteDrawingBrush As SolidBrush = New SolidBrush(BackgroundColor)

    Dim DimGrayDrawingPen As New Pen(Color.DimGray, 1)
    Dim DimGrayDrawingBrush As SolidBrush = New SolidBrush(Color.DimGray)

    Dim BlueGraphDrawingPen As New Pen(Color.Blue, 1)
    Dim BlueGraphDrawingBrush As SolidBrush = New SolidBrush(Color.Blue)
    Dim MagentaGraphDrawingPen As New Pen(Color.Magenta, 1)
    Dim MagentaGraphDrawingBrush As SolidBrush = New SolidBrush(Color.Magenta)
    Dim RedGraphDrawingPen As New Pen(Color.Red, 1)
    Dim RedGraphDrawingBrush As SolidBrush = New SolidBrush(Color.Red)
    Dim GreenGraphDrawingPen As New Pen(Color.Green, 1)
    Dim GreenGraphDrawingBrush As SolidBrush = New SolidBrush(Color.Green)
    Dim OrangeGraphDrawingPen As New Pen(Color.Orange, 1)
    Dim OrangeGraphDrawingBrush As SolidBrush = New SolidBrush(Color.Orange)

    Dim FPSDrawingBrush As SolidBrush = New SolidBrush(Color.Red)
    Dim BackgroundDrawingBrush As SolidBrush = New SolidBrush(Color.Red)
    Dim SkyBlueDrawingBrush As SolidBrush = New SolidBrush(Color.SkyBlue)
    Dim tempSize As SizeF
    Public ConstTempString As String
    Public ConstTempSize As SizeF
    Dim DrawingObjectRectangle As Rectangle

    Dim Min, Max As Integer
    Public DebugString As String

    Public Sub UpdateEllapsedTime()
        Static oldTicks As Long

        If oldTicks = 0 Then
            oldTicks = System.Environment.TickCount
        Else
            EllapsedTime = Math.Max(1, System.Environment.TickCount - oldTicks)
            oldTicks = System.Environment.TickCount
        End If

    End Sub

    Public Sub CreateBackBufferDraw()
        'If it is "Nothing" already no reset required
        If Not BackBufferDrawBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            BackBufferDrawBitMap.Dispose()
            BackBufferDraw.Dispose()
            BackBufferDrawBitMap = Nothing
            BackBufferDraw = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        BackBufferDrawBitMap = New Bitmap(frmFuel.pctDraw.Width, frmFuel.pctDraw.Height)
        BackBufferDraw = Graphics.FromImage(BackBufferDrawBitMap)
    End Sub

    Public Sub CreatePreBufferMenu()
        'If it is "Nothing" already no reset required
        If Not MenuBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            MenuBitMap.Dispose()
            MenuDraw.Dispose()
            MenuBitMap = Nothing
            MenuDraw = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        MenuBitMap = New Bitmap(Math.Max(Math.Ceiling(MenuDrawSize.Width / (MenuItemCount)), IconSize.Width + 1) * (MenuLoc.EndMenu + 1), MenuDrawSize.Height)
        MenuDraw = Graphics.FromImage(MenuBitMap)
        MenuDraw.Clear(BackgroundColor)
    End Sub

    Public Sub CreateBackBuffer()
        'If it is "Nothing" already no reset required
        If Not BackBufferBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            BackBufferBitMap.Dispose()
            BackBuffer.Dispose()
            BackBufferBitMap = Nothing
            BackBuffer = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        BackBufferBitMap = New Bitmap(InnerDrawSize.Width, Math.Max(ObjectHeight(CurrentLocation) * (ItemMax(CurrentLocation) + 1), InnerDrawSize.Height))
        BackBuffer = Graphics.FromImage(BackBufferBitMap)
    End Sub

    Public Sub CreateOwnBackBuffer(ByVal ID As MenuLoc, ByVal Height As Integer)
        'If it is "Nothing" already no reset required
        If Not BufferBitMap(ID) Is Nothing Then
            'Clear RAM and map it to nothing
            BufferBitMap(ID).Dispose()
            Buffer.Dispose()
            BufferBitMap(ID) = Nothing
            Buffer = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        BufferBitMap(ID) = New Bitmap(InnerDrawSize.Width, Math.Max(Height, InnerDrawSize.Height))
        Buffer = Graphics.FromImage(BufferBitMap(ID))
        Buffer.Clear(BackgroundColor)
    End Sub

    Public Sub CreateBackBufferMenu()
        'If it is "Nothing" already no reset required
        If Not BackBufferMenuBitMap Is Nothing Then
            'Clear RAM and map it to nothing
            BackBufferMenuBitMap.Dispose()
            BackBufferMenu.Dispose()
            BackBufferMenuBitMap = Nothing
            BackBufferMenu = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        BackBufferMenuBitMap = New Bitmap(MenuDrawSize.Width, MenuDrawSize.Height)
        BackBufferMenu = Graphics.FromImage(BackBufferMenuBitMap)
    End Sub

    Public Sub CreateBackBufferIcon(ByVal ID As Byte)
        'If it is "Nothing" already no reset required
        If Not MenuIconBitMap(ID) Is Nothing Then
            'Clear RAM and map it to nothing
            MenuIconBitMap(ID).Dispose()
            MenuIcon.Dispose()
            MenuIconBitMap(ID) = Nothing
            MenuIcon = Nothing
        End If
        'Create a Backbuffer against ugly flickering
        MenuIconBitMap(ID) = New Bitmap(IconSize.Width, IconSize.Height)
        MenuIcon = Graphics.FromImage(MenuIconBitMap(ID))
        MenuIcon.Clear(BackgroundColor)
    End Sub

    Public Sub simpleDrawTest()
        Dim I As Integer = 0
        Dim TempString As String = ""

        'Drawing Routine
        Min = Math.Max(1, Int(-CurrentPosition(CurrentLocation).Y / ObjectHeight(CurrentLocation)))
        Max = Math.Min(ItemMax(CurrentLocation), Int(-CurrentPosition(CurrentLocation).Y / ObjectHeight(CurrentLocation) + frmFuel.Height / ObjectHeight(CurrentLocation)))
        NeedRedraw = False
        'Create our anti-flicker buffer
        'CreateBackBuffer()
        'Set background color
        BackBuffer.FillRectangle(New SolidBrush(Color.WhiteSmoke), 0, 0, BackBufferBitMap.Width, BackBufferBitMap.Height)
        'This Routine is intelligent so it only draws the objects which are seen
        'If it would draw every of for ex. 2000 objects when only 7 are visible, it would get very slow and 
        'also why draw something which you cannot see?

        If Min = 1 Then
            DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + BlackDrawingPen.Width, frmFuel.pctDraw.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(CurrentLocation) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))
            'Get Size of the text to center it
            'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
            BackBuffer.FillRectangle(BlackDrawingBrush, DrawingObjectRectangle)
            BackBuffer.DrawString(ConstTempString, frmFuel.Font, WhiteDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            TempString = Stats.All.KiloMeters & "km [" & Stats.Average.KiloMeters & "] (" & Stats.MinMax.Min.KiloMeters & "/" & Stats.MinMax.Max.KiloMeters & ")" & vbNewLine & _
                         Stats.All.FuelAmount & "l [" & Stats.Average.FuelAmount & "] (" & Stats.MinMax.Min.FuelAmount & "/" & Stats.MinMax.Max.FuelAmount & ")" & vbNewLine & _
                         Stats.All.PriceAmount & " [" & Stats.Average.PriceAmount & "] (" & Stats.MinMax.Min.PriceAmount & "/" & Stats.MinMax.Max.PriceAmount & ")" & vbNewLine & _
                         Stats.Average.PricePerLiter & " (" & Stats.MinMax.Min.PricePerLiter & "/" & Stats.MinMax.Max.PricePerLiter & ")" & vbNewLine & _
                         Stats.Average.ConsumptionPer100 & " (" & Stats.MinMax.Min.ConsumptionPer100 & "/" & Stats.MinMax.Max.ConsumptionPer100 & ")"
            'BackBuffer.DrawString(TempString, frmFuel.Font, BlackDrawingBrush, (frmFuel.pctDraw.Width / 2) - (tempSize.Width / 2), I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)
            'Draw Icon if it is not emtpy so it will be skipped if the Bitmap isn't already loaded
            BackBuffer.DrawString(TempString, frmFuel.Font, WhiteDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator) + ConstTempSize.Width, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            'TempString = "Average"
            'tempSize = BackBuffer.MeasureString(TempString, frmFuel.Font)
            'BackBuffer.DrawString(TempString, frmFuel.Font, WhiteDrawingBrush, frmFuel.pctDraw.Width - tempSize.Width - 6, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)

        End If
        Dim smallI As Integer = 0
        For I = Min To Max
            smallI = I - 1

            DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + BlackDrawingPen.Width, frmFuel.pctDraw.Width - BlackDrawingPen.Width * 4, ObjectHeight(CurrentLocation) - BlackDrawingPen.Width * 4)
            'Get Size of the text to center it
            'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
            BackBuffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)
            BackBuffer.DrawString(ConstTempString, frmFuel.Font, BlackDrawingBrush, 6, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            TempString = FuelList(smallI).KiloMeters & "km" & vbNewLine & _
                         FuelList(smallI).FuelAmount & "l" & vbNewLine & _
                         FuelList(smallI).PriceAmount & vbNewLine & _
                         FuelList(smallI).PricePerLiter & vbNewLine & _
                         FuelList(smallI).ConsumptionPer100 & "l"
            BackBuffer.DrawString(TempString, frmFuel.Font, BlackDrawingBrush, 6 + ConstTempSize.Width, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            TempString = I & ". " '& FuelList(smallI).TimePrint
            tempSize = BackBuffer.MeasureString(TempString, frmFuel.Font)
            BackBuffer.DrawString(TempString, frmFuel.Font, FPSDrawingBrush, frmFuel.pctDraw.Width - tempSize.Width - 6, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)
            'Draw Icon if it is not emtpy so it will be skipped if the Bitmap isn't already loaded
        Next
        'Instead of Using the Same Item everytime you could use an array instead of items ;)
        'BackBuffer.DrawString(FPS, frmFuel.Font, FPSDrawingBrush, BackBufferBitMap.Width - 5 - BackBuffer.MeasureString(FPS, frmFuel.Font).Width, 10)
        'BackBuffer.DrawString(DebugString & (Speed), frmFuel.Font, FPSDrawingBrush, BackBufferBitMap.Width - 5 - BackBuffer.MeasureString(DebugString & (Speed), frmFuel.Font).Width, BackBufferBitMap.Height - 5 - BackBuffer.MeasureString(DebugString & (Speed), frmFuel.Font).Height)
        'Just * 1000 for pixel per second
        NeedRefresh = True
    End Sub

    Public Sub PreDrawIcons()
        Dim IconCount As MenuLoc
        MenuDrawSize.Height = 48 * Config(MyConfig.ScreenMultiplicator)
        IconSize.Height = MenuDrawSize.Height
        IconSize.Width = MenuDrawSize.Height
        Dim DrawBrush As New SolidBrush(ForegroundColor)
        Dim DrawPen As New Pen(ForegroundColor, IconSize.Width / 16)
        Dim DrawThinPen As New Pen(ForegroundColor, IconSize.Width / 32)
        Dim TempPath As String

        'Stats
        IconCount = MenuLoc.Stats
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Stats" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                .FillEllipse(DrawBrush, New Rectangle(IconSize.Width / 16, IconSize.Height / 16, 7 * IconSize.Width / 8, 7 * IconSize.Height / 8))
                .FillEllipse(New SolidBrush(BackgroundColor), New Rectangle(IconSize.Width / 8, IconSize.Height / 8, 3 * IconSize.Width / 4, 3 * IconSize.Height / 4))
                .DrawLine(DrawPen, 0, 0, IconSize.Width, IconSize.Height)
            End If
        End With
        'Graph
        IconCount = MenuLoc.Graph
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Graphs" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                .DrawLine(DrawPen, IconSize.Width / 16, IconSize.Height / 16, IconSize.Width / 16, 15 * IconSize.Height / 16)
                .DrawLine(DrawPen, IconSize.Width / 16, 15 * IconSize.Height / 16, 15 * IconSize.Width / 16, 15 * IconSize.Height / 16)
                .DrawLine(DrawThinPen, IconSize.Width / 16, 15 * IconSize.Height / 16, IconSize.Width / 4, IconSize.Height / 8)
                .DrawLine(DrawThinPen, IconSize.Width / 4, IconSize.Height / 8, 3 * IconSize.Width / 8, IconSize.Height / 2)
                .DrawLine(DrawThinPen, 3 * IconSize.Width / 8, IconSize.Height / 2, 5 * IconSize.Width / 8, 1 * IconSize.Height / 4)
                .DrawLine(DrawThinPen, 5 * IconSize.Width / 8, 1 * IconSize.Height / 4, 7 * IconSize.Width / 8, 3 * IconSize.Height / 4)
            End If
        End With
        'All
        IconCount = MenuLoc.All
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "List" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                .DrawRectangle(DrawPen, IconSize.Width / 16, IconSize.Height / 16, 15 * IconSize.Width / 16, 15 * IconSize.Height / 16)
                For I = 1 To 4
                    .DrawLine(DrawThinPen, 3 * IconSize.Width / 16, I * 3 * IconSize.Height / 16, 13 * IconSize.Width / 16, I * 3 * IconSize.Height / 16)
                Next
            End If
        End With
        'Add
        IconCount = MenuLoc.Add
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Add" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                .DrawLine(DrawPen, IconSize.Width / 16, IconSize.Height / 2, 15 * IconSize.Width / 16, IconSize.Height / 2)
                .DrawLine(DrawPen, IconSize.Width / 2, 15 * IconSize.Height / 16, IconSize.Width / 2, IconSize.Height / 16)
            End If
        End With
        'Settings
        IconCount = MenuLoc.Config
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Config" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                Dim ConfigLetterString As String = "C"
                Dim ConfigLetterSize As SizeF = .MeasureString(ConfigLetterString, frmFuel.Font)
                Dim NewFont As New Font(frmFuel.Font.Name, Math.Min((frmFuel.Font.Size / ConfigLetterSize.Width) * IconSize.Width, (frmFuel.Font.Size / ConfigLetterSize.Height) * IconSize.Height) * 0.9, FontStyle.Bold)
                ConfigLetterSize = .MeasureString(ConfigLetterString, NewFont)
                .DrawString(ConfigLetterString, NewFont, BlackDrawingBrush, (IconSize.Width / 2) - (ConfigLetterSize.Width / 2), (IconSize.Height / 2) - (ConfigLetterSize.Height / 2))
            End If
        End With
        'About
        IconCount = MenuLoc.About
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Help" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                Dim QuestionMarkString As String = "?"
                Dim QuestionMarkSize As SizeF = .MeasureString(QuestionMarkString, frmFuel.Font)
                Dim NewFont As New Font(frmFuel.Font.Name, Math.Min((frmFuel.Font.Size / QuestionMarkSize.Width) * IconSize.Width, (frmFuel.Font.Size / QuestionMarkSize.Height) * IconSize.Height) * 0.9, FontStyle.Bold)
                QuestionMarkSize = .MeasureString(QuestionMarkString, NewFont)
                .DrawString(QuestionMarkString, NewFont, BlackDrawingBrush, (IconSize.Width / 2) - (QuestionMarkSize.Width / 2), (IconSize.Height / 2) - (QuestionMarkSize.Height / 2))
            End If
        End With
        'Exit
        IconCount = MenuLoc.EndMenu
        Call CreateBackBufferIcon(IconCount)
        With MenuIcon
            TempPath = AppPath & "\gfx\" & Config(MyConfig.IconSet) & "\" & Config(MyConfig.IconSet) & "-" & "Menu" & "Exit" & IconSize.Height & ".jpg"
            If System.IO.File.Exists(TempPath) Then
                .DrawImage(New Bitmap(TempPath), 0, 0)
            Else
                .DrawLine(DrawPen, IconSize.Width / 16, IconSize.Height / 16, 15 * IconSize.Width / 16, 15 * IconSize.Height / 16)
                .DrawLine(DrawPen, IconSize.Width / 16, 15 * IconSize.Height / 16, 15 * IconSize.Width / 16, IconSize.Height / 16)
            End If
        End With
    End Sub

    Public Sub PreDrawMenu()
        Call CreatePreBufferMenu()
        For I = 0 To MenuItemCount - 1
            'MenuDraw.DrawImage(MenuIconBitMap(I), I * (Math.Max(Math.Ceiling(MenuDrawSize.Width / (MenuItemCount)), IconSize.Width + 1)) + (Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1)) / 2 - (IconSize.Width / 2), 0)
            MenuDraw.DrawImage(MenuIconBitMap(I), I * Math.Max(MenuDrawSize.Width / MenuItemCount, IconSize.Width) + Math.Max(MenuDrawSize.Width / MenuItemCount - IconSize.Width, 0) / 2, 0) 'I * (IconSize.Width), 0)
        Next
        'For I = 0 To MenuItemCount - 1
        '    MenuDraw.DrawLine(BlackDrawingPen, I * (Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1)) + (Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1)), 0, I * (Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1)) + (Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1)), MenuDrawSize.Height)
        'Next
    End Sub

    Public Sub DrawMenu()
        Dim PositionCorrection As Integer = (-CurrentLocation / (MenuItemCount - 1)) * (Math.Max(MenuDrawSize.Width, (IconSize.Width) * (MenuItemCount)) - (MenuDrawSize.Width))
        Call CreateBackBufferMenu()
        BackBufferMenu.DrawImage(MenuBitMap, PositionCorrection, 0)
        BackBufferMenu.DrawRectangle(New Pen(Color.Gray, MenuDrawSize.Height / 8), PositionCorrection + Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width) * CurrentLocation, 0, Math.Max(MenuDrawSize.Width / (MenuItemCount), IconSize.Width + 1), MenuDrawSize.Height)
    End Sub

    Public Sub OldPreDraw()
        Dim I As Integer = 0
        Dim TempString As String
        Dim GraphSpacing As SizeF
        Dim GraphFactor As Double
        Call CreateBackBuffer()
        'Drawing Routine
        Min = 1 + 1
        Max = ItemMax(CurrentLocation)
        NeedRedraw = False
        'Create our anti-flicker buffer
        'CreateBackBuffer()
        'Set background color
        BackBuffer.FillRectangle(New SolidBrush(Color.WhiteSmoke), 0, 0, BackBufferBitMap.Width, BackBufferBitMap.Height)
        'This Routine is intelligent so it only draws the objects which are seen
        'If it would draw every of for ex. 2000 objects when only 7 are visible, it would get very slow and 
        'also why draw something which you cannot see?

        DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2, I * ObjectHeight(CurrentLocation) + BlackDrawingPen.Width, frmFuel.pctDraw.Width - BlackDrawingPen.Width * 4, ObjectHeight(CurrentLocation) - BlackDrawingPen.Width * 4)
        'Get Size of the text to center it
        'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
        BackBuffer.FillRectangle(BlackDrawingBrush, DrawingObjectRectangle)
        BackBuffer.DrawString(ConstTempString, frmFuel.Font, WhiteDrawingBrush, 6, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

        TempString = Stats.All.KiloMeters & Config(MyConfig.MetricsUnit) & " [" & Stats.Average.KiloMeters & "] (" & Stats.MinMax.Min.KiloMeters & "/" & Stats.MinMax.Max.KiloMeters & ")" & vbNewLine & _
                     Stats.All.FuelAmount & "l [" & Stats.Average.FuelAmount & "] (" & Stats.MinMax.Min.FuelAmount & "/" & Stats.MinMax.Max.FuelAmount & ")" & vbNewLine & _
                     Stats.All.PriceAmount & Config(MyConfig.CurrencyUnit) & " [" & Stats.Average.PriceAmount & "] (" & Stats.MinMax.Min.PriceAmount & "/" & Stats.MinMax.Max.PriceAmount & ")" & vbNewLine & _
                     Stats.Average.PricePerLiter & Config(MyConfig.CurrencyUnit) & " (" & Stats.MinMax.Min.PricePerLiter & "/" & Stats.MinMax.Max.PricePerLiter & ")" & vbNewLine & _
                     Stats.Average.ConsumptionPer100 & " (" & Stats.MinMax.Min.ConsumptionPer100 & "/" & Stats.MinMax.Max.ConsumptionPer100 & ")" & vbNewLine & _
                     Stats.All.TimeLength & "days [" & Stats.Average.TimeDiff & " days]"
        'BackBuffer.DrawString(TempString, frmFuel.Font, BlackDrawingBrush, (frmFuel.pctDraw.Width / 2) - (tempSize.Width / 2), I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)
        'Draw Icon if it is not emtpy so it will be skipped if the Bitmap isn't already loaded
        BackBuffer.DrawString(TempString, frmFuel.Font, WhiteDrawingBrush, 6 + ConstTempSize.Width, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

        'TempString = "Average"
        'tempSize = BackBuffer.MeasureString(TempString, frmFuel.Font)
        'BackBuffer.DrawString(TempString, frmFuel.Font, WhiteDrawingBrush, frmFuel.pctDraw.Width - tempSize.Width - 6, I * ObjectHeight(CurrentLocation) + CurrentPosition(CurrentLocation).Y + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)

        'GRAPH!

        BackBuffer.FillEllipse(RedGraphDrawingBrush, New Rectangle(frmFuel.pctDraw.Width - 12, 0 * ConstTempSize.Height / 6 + ConstTempSize.Height / 12, 6, 6))
        BackBuffer.FillEllipse(BlueGraphDrawingBrush, New Rectangle(frmFuel.pctDraw.Width - 12, 1 * ConstTempSize.Height / 6 + ConstTempSize.Height / 12, 6, 6))
        BackBuffer.FillEllipse(OrangeGraphDrawingBrush, New Rectangle(frmFuel.pctDraw.Width - 12, 2 * ConstTempSize.Height / 6 + ConstTempSize.Height / 12, 6, 6))
        BackBuffer.FillEllipse(GreenGraphDrawingBrush, New Rectangle(frmFuel.pctDraw.Width - 12, 3 * ConstTempSize.Height / 6 + ConstTempSize.Height / 12, 6, 6))
        BackBuffer.FillEllipse(MagentaGraphDrawingBrush, New Rectangle(frmFuel.pctDraw.Width - 12, 4 * ConstTempSize.Height / 6 + ConstTempSize.Height / 12, 6, 6))

        I += 1
        DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2, I * ObjectHeight(CurrentLocation) + BlackDrawingPen.Width, frmFuel.pctDraw.Width - BlackDrawingPen.Width * 4, ObjectHeight(CurrentLocation) - BlackDrawingPen.Width * 4)
        'Get Size of the text to center it
        'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
        BackBuffer.FillRectangle(BlackDrawingBrush, DrawingObjectRectangle)
        BackBuffer.DrawLine(WhiteDrawingPen, 6, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) - 12, frmFuel.pctDraw.Width - 12, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) - 12)
        BackBuffer.DrawLine(WhiteDrawingPen, 12, I * ObjectHeight(CurrentLocation) + 6, 12, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) - 6)

        If ItemMax(CurrentLocation) > 2 Then
            GraphSpacing.Width = (frmFuel.pctDraw.Width - 24) / (ItemMax(CurrentLocation) - 2)
            GraphSpacing.Height = ObjectHeight(CurrentLocation) - 24
            GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.KiloMeters)

            For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                BackBuffer.DrawLine(DimGrayDrawingPen, J * GraphSpacing.Width + 12, I * ObjectHeight(CurrentLocation) + 6, J * GraphSpacing.Width + 12, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) - 6)
            Next

            If ShowGraph(0) = True Then
                For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                    BackBuffer.DrawLine(RedGraphDrawingPen, J * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J).KiloMeters * GraphFactor - 12, (J - 1) * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J - 1).KiloMeters * GraphFactor - 12)
                Next
                'BackBuffer.DrawLine(RedGraphDrawingPen, 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.KiloMeters * GraphFactor - 12, frmFuel.pctDraw.Width - 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.KiloMeters * GraphFactor - 12)
            End If

            If ShowGraph(1) = True Then
                GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.FuelAmount)
                For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                    BackBuffer.DrawLine(BlueGraphDrawingPen, J * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J).FuelAmount * GraphFactor - 12, (J - 1) * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J - 1).FuelAmount * GraphFactor - 12)
                Next
                'BackBuffer.DrawLine(BlueGraphDrawingPen, 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.FuelAmount * GraphFactor - 12, frmFuel.pctDraw.Width - 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.FuelAmount * GraphFactor - 12)
            End If

            If ShowGraph(2) = True Then
                GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.PriceAmount)
                For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                    BackBuffer.DrawLine(OrangeGraphDrawingPen, J * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J).PriceAmount * GraphFactor - 12, (J - 1) * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J - 1).PriceAmount * GraphFactor - 12)
                Next
                'BackBuffer.DrawLine(OrangeGraphDrawingPen, 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.PriceAmount * GraphFactor - 12, frmFuel.pctDraw.Width - 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.PriceAmount * GraphFactor - 12)
            End If

            If ShowGraph(3) = True Then
                GraphFactor = GraphSpacing.Height / Math.Max(0.001, Stats.MinMax.Max.PricePerLiter)
                For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                    BackBuffer.DrawLine(GreenGraphDrawingPen, J * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J).PricePerLiter * GraphFactor - 12, (J - 1) * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J - 1).PricePerLiter * GraphFactor - 12)
                Next
                'BackBuffer.DrawLine(GreenGraphDrawingPen, 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.PricePerLiter * GraphFactor - 12, frmFuel.pctDraw.Width - 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.PricePerLiter * GraphFactor - 12)
            End If

            If ShowGraph(4) = True Then
                GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.ConsumptionPer100)
                For J = ItemMax(CurrentLocation) - 2 To 1 Step -1
                    BackBuffer.DrawLine(MagentaGraphDrawingPen, J * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J).ConsumptionPer100 * GraphFactor - 12, (J - 1) * GraphSpacing.Width + 12, (I + 1) * ObjectHeight(CurrentLocation) - FuelList(J - 1).ConsumptionPer100 * GraphFactor - 12)
                Next
                'BackBuffer.DrawLine(MagentaGraphDrawingPen, 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.ConsumptionPer100 * GraphFactor - 12, frmFuel.pctDraw.Width - 12, (I + 1) * ObjectHeight(CurrentLocation) - Stats.Average.ConsumptionPer100 * GraphFactor - 12)
            End If
        End If



        Dim smallI As Integer = 0
        For I = Min To Max
            smallI = I - 2

            DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2, I * ObjectHeight(CurrentLocation) + BlackDrawingPen.Width, frmFuel.pctDraw.Width - BlackDrawingPen.Width * 4, ObjectHeight(CurrentLocation) - BlackDrawingPen.Width * 4)
            'Get Size of the text to center it
            'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
            BackBuffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)
            BackBuffer.DrawString(ConstTempString, frmFuel.Font, BlackDrawingBrush, 6, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            TempString = FuelList(smallI).KiloMeters & Config(MyConfig.MetricsUnit) & vbNewLine & _
                         FuelList(smallI).FuelAmount & "l" & vbNewLine & _
                         FuelList(smallI).PriceAmount & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                         FuelList(smallI).PricePerLiter & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                         FuelList(smallI).ConsumptionPer100 & "l" & vbNewLine & _
                         FuelList(smallI).TimePrint

            BackBuffer.DrawString(TempString, frmFuel.Font, BlackDrawingBrush, 6 + ConstTempSize.Width, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) / 2 - ConstTempSize.Height / 2)

            TempString = (I - 1) & ". " '& FuelList(smallI).TimePrint
            tempSize = BackBuffer.MeasureString(TempString, frmFuel.Font)
            BackBuffer.DrawString(TempString, frmFuel.Font, FPSDrawingBrush, frmFuel.pctDraw.Width - tempSize.Width - 6, I * ObjectHeight(CurrentLocation) + ObjectHeight(CurrentLocation) / 2 - tempSize.Height / 2)
            'Draw Icon if it is not emtpy so it will be skipped if the Bitmap isn't already loaded
        Next
        'Instead of Using the Same Item everytime you could use an array instead of items ;)
        'BackBuffer.DrawString(FPS, frmFuel.Font, FPSDrawingBrush, BackBufferBitMap.Width - 5 - BackBuffer.MeasureString(FPS, frmFuel.Font).Width, 10)
        'BackBuffer.DrawString(DebugString & (Speed), frmFuel.Font, FPSDrawingBrush, BackBufferBitMap.Width - 5 - BackBuffer.MeasureString(DebugString & (Speed), frmFuel.Font).Width, BackBufferBitMap.Height - 5 - BackBuffer.MeasureString(DebugString & (Speed), frmFuel.Font).Height)
        'Just * 1000 for pixel per second
        NeedRefresh = True
    End Sub

    Public Function Hex2RGB(ByVal HexColor As String) As Color
        Dim Red, Green, Blue As String

        HexColor = Replace(HexColor, "#", "")

        Red = Val("&H" & Mid(HexColor, 1, 2))
        Green = Val("&H" & Mid(HexColor, 3, 2))
        Blue = Val("&H" & Mid(HexColor, 5, 2))

        Return Color.FromArgb(Red, Green, Blue)
    End Function

    Public Sub PreDraw()
        ForegroundColor = Hex2RGB(Config(MyConfig.ForegroundColor))
        BackgroundColor = Hex2RGB(Config(MyConfig.BackgroundColor))

        BlackDrawingPen = New Pen(ForegroundColor, 1)
        BlackDrawingBrush = New SolidBrush(ForegroundColor)
        WhiteDrawingPen = New Pen(BackgroundColor, 1)
        WhiteDrawingBrush = New SolidBrush(BackgroundColor)

        Call PreDrawMenu()
        Call PreDrawStats()
        Call PreDrawGraphs()
        Call PreDrawAll()
        Call PreDrawAdd()
        Call PreDrawConfig()
        Call PreDrawAbout()
        Call PreDrawEndMenu()
    End Sub

    Public Sub PreDrawStats()
        Call CreateOwnBackBuffer(MenuLoc.Stats, StatTextSize.Height)
        Buffer.DrawString(StatText, frmFuel.Font, BlackDrawingBrush, 2, FontLineHeight * 2)

        Dim TempString(,) As String = {{"All", "Avr.", "Min", "Max"}, _
                                       {Stats.All.KiloMeters, Stats.Average.KiloMeters, Stats.MinMax.Min.KiloMeters, Stats.MinMax.Max.KiloMeters}, _
                                       {Stats.All.FuelAmount, Stats.Average.FuelAmount, Stats.MinMax.Min.FuelAmount, Stats.MinMax.Max.FuelAmount}, _
                                       {Stats.All.PriceAmount, Stats.Average.PriceAmount, Stats.MinMax.Min.PriceAmount, Stats.MinMax.Max.PriceAmount}, _
                                       {"", Stats.Average.PricePerLiter, Stats.MinMax.Min.PricePerLiter, Stats.MinMax.Max.PricePerLiter}, _
                                       {"", Stats.Average.ConsumptionPer100, Stats.MinMax.Min.ConsumptionPer100, Stats.MinMax.Max.ConsumptionPer100}, _
                                       {"", Stats.Average.MilesPerGallon, Stats.MinMax.Min.MilesPerGallon, Stats.MinMax.Max.MilesPerGallon}, _
                                       {Stats.All.TimeLength & "d", Stats.Average.TimeDiff & "d", Stats.MinMax.Min.TimeDiff & "d", Stats.MinMax.Max.TimeDiff & "d"}, _
                                       {"", Stats.Average.LeftKiloMeters, Stats.MinMax.Min.LeftKiloMeters, Stats.MinMax.Max.LeftKiloMeters}, _
                                       {"", Stats.Average.LeftFuelAmount, Stats.MinMax.Min.LeftFuelAmount, Stats.MinMax.Max.LeftFuelAmount}, _
                                       {"", Stats.Average.LeftTimeLength & "d", Stats.MinMax.Min.LeftTimeLength & "d", Stats.MinMax.Max.LeftTimeLength & "d"}, _
                                       {"All", "Avr.", "Min", "Max"} _
                                      }
        Dim TempSize As SizeF
        Dim TempSpacing As Integer

        For J = 0 To TempString.GetLength(0) - 1
            For I = 0 To TempString.GetLength(1) - 1
                TempSize = Buffer.MeasureString(TempString(J, I), frmFuel.Font)
                TempSpacing = ((StatTextSize.Width + 4 * Config(MyConfig.ScreenMultiplicator) + ((InnerDrawSize.Width - StatTextSize.Width - 4 * Config(MyConfig.ScreenMultiplicator)) / 4) * (I + 1)) - (StatTextSize.Width + 4 * Config(MyConfig.ScreenMultiplicator) + ((InnerDrawSize.Width - StatTextSize.Width - 4 * Config(MyConfig.ScreenMultiplicator)) / 4) * (I))) / 2 + (StatTextSize.Width + 4 * Config(MyConfig.ScreenMultiplicator) + ((InnerDrawSize.Width - StatTextSize.Width - 4 * Config(MyConfig.ScreenMultiplicator)) / 4) * (I)) - (TempSize.Width / 2)
                Buffer.DrawString(TempString(J, I), frmFuel.Font, BlackDrawingBrush, TempSpacing, 2 + J * FontLineHeight * 2)
            Next
        Next

        For I = 1 To TempString.GetLength(0) - 1
            Buffer.DrawLine(BlackDrawingPen, 2 * Config(MyConfig.ScreenMultiplicator), FontLineHeight * 2 * I - (FontLineHeight / 2), InnerDrawSize.Width - 2 * Config(MyConfig.ScreenMultiplicator), FontLineHeight * 2 * I - (FontLineHeight / 2))
        Next

        For I = 0 To TempString.GetLength(1) - 1
            Buffer.DrawLine(BlackDrawingPen, StatTextSize.Width + 4 * Config(MyConfig.ScreenMultiplicator) + ((InnerDrawSize.Width - StatTextSize.Width - 4 * Config(MyConfig.ScreenMultiplicator)) / 4) * I, 2 * Config(MyConfig.ScreenMultiplicator), StatTextSize.Width + 4 * Config(MyConfig.ScreenMultiplicator) + ((InnerDrawSize.Width - StatTextSize.Width - 4 * Config(MyConfig.ScreenMultiplicator)) / 4) * I, StatTextSize.Height + 2 * Config(MyConfig.ScreenMultiplicator))
        Next
    End Sub

    Public Sub PreDrawGraphs()
        Call CreateOwnBackBuffer(MenuLoc.Graph, InnerDrawSize.Height)

        Dim GraphDrawPen() As Pen = {RedGraphDrawingPen, BlueGraphDrawingPen, OrangeGraphDrawingPen, GreenGraphDrawingPen, MagentaGraphDrawingPen}

        Dim GraphSpacing As SizeF
        Dim GraphFactor As Double
        Dim GraphCount As Integer = 0
        If MaxGraphItems = 0 Or MaxGraphItems >= FuelList.Length Then
            MaxGraphItems = FuelList.Length - 1
        End If

        ButtonDrawingRightSize.Width = 36 * Config(MyConfig.ScreenMultiplicator)
        ButtonDrawingBottomSize.Height = 36 * Config(MyConfig.ScreenMultiplicator)
        GraphDrawingSize.Width = InnerDrawSize.Width - ButtonDrawingRightSize.Width
        GraphDrawingSize.Height = InnerDrawSize.Height - ButtonDrawingBottomSize.Height

        If GraphDrawingSize.Height > GraphDrawingSize.Width Then
            ButtonDrawingRightSize.Height = GraphDrawingSize.Height - 2 * Config(MyConfig.ScreenMultiplicator)
            ButtonDrawingBottomSize.Width = InnerDrawSize.Width
        Else
            ButtonDrawingRightSize.Height = InnerDrawSize.Height
            ButtonDrawingBottomSize.Width = GraphDrawingSize.Width - 2 * Config(MyConfig.ScreenMultiplicator)
        End If

        ButtonRightItemHeight = ButtonDrawingRightSize.Height / 5
        ButtonBottomItemWidth = ButtonDrawingBottomSize.Width / 5

        Dim TextBrush As SolidBrush
        For I = 0 To 4
            If (VisibleCountArray(I, ConfigType.Value) = MaxGraphItems And Not MaxGraphItems = ItemMax(MenuLoc.All)) Or (I = 4 And MaxGraphItems = ItemMax(MenuLoc.All)) Then
                Buffer.DrawRectangle(BlackDrawingPen, GraphDrawingSize.Width, I * ButtonRightItemHeight + 2 * Config(MyConfig.ScreenMultiplicator), ButtonDrawingRightSize.Width - 2 * Config(MyConfig.ScreenMultiplicator), ButtonRightItemHeight - 4 * Config(MyConfig.ScreenMultiplicator))
                TextBrush = BlackDrawingBrush
            Else
                Buffer.FillRectangle(BlackDrawingBrush, GraphDrawingSize.Width, I * ButtonRightItemHeight + 2 * Config(MyConfig.ScreenMultiplicator), ButtonDrawingRightSize.Width - 2 * Config(MyConfig.ScreenMultiplicator), ButtonRightItemHeight - 4 * Config(MyConfig.ScreenMultiplicator))
                TextBrush = WhiteDrawingBrush
            End If
            Buffer.DrawString(VisibleCountArray(I, ConfigType.Text), frmFuel.Font, TextBrush, GraphDrawingSize.Width + ButtonDrawingRightSize.Width / 2 - VisibleCountSize(I).Width / 2, I * ButtonRightItemHeight + ButtonRightItemHeight / 2 - VisibleCountSize(I).Height / 2)

            If ShowGraph(I) = True Then
                Buffer.DrawRectangle(BlackDrawingPen, I * ButtonBottomItemWidth + 1 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Height, ButtonBottomItemWidth - 2 * Config(MyConfig.ScreenMultiplicator), ButtonDrawingBottomSize.Width - 1 * Config(MyConfig.ScreenMultiplicator))
                TextBrush = BlackDrawingBrush
            Else
                Buffer.FillRectangle(BlackDrawingBrush, I * ButtonBottomItemWidth + 1 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Height, ButtonBottomItemWidth - 2 * Config(MyConfig.ScreenMultiplicator), ButtonDrawingBottomSize.Width - 1 * Config(MyConfig.ScreenMultiplicator))
                TextBrush = WhiteDrawingBrush
            End If
            Buffer.DrawEllipse(GraphDrawPen(I), I * ButtonBottomItemWidth + 4 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Height + 4 * Config(MyConfig.ScreenMultiplicator), 4 * Config(MyConfig.ScreenMultiplicator), 4 * Config(MyConfig.ScreenMultiplicator))
            Buffer.DrawEllipse(GraphDrawPen(I), I * ButtonBottomItemWidth + 5 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Height + 5 * Config(MyConfig.ScreenMultiplicator), 2 * Config(MyConfig.ScreenMultiplicator), 2 * Config(MyConfig.ScreenMultiplicator))
            Buffer.DrawString(VisibleGraphArray(I), frmFuel.Font, TextBrush, I * ButtonBottomItemWidth + ButtonBottomItemWidth / 2 - VisibleGraphSize(I).Width / 2, GraphDrawingSize.Height + ButtonDrawingBottomSize.Height / 2 - VisibleGraphSize(I).Height / 2)
        Next

        Buffer.FillRectangle(WhiteDrawingBrush, 0, 0, GraphDrawingSize.Width, GraphDrawingSize.Height)
        Buffer.DrawLine(BlackDrawingPen, 6 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + GraphDrawingSize.Height - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + GraphDrawingSize.Height - 12 * Config(MyConfig.ScreenMultiplicator))
        Buffer.DrawLine(BlackDrawingPen, 12 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + 6 * Config(MyConfig.ScreenMultiplicator), 12 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + GraphDrawingSize.Height - 6 * Config(MyConfig.ScreenMultiplicator))
        If MaxGraphItems > 0 Then
            GraphSpacing.Width = (GraphDrawingSize.Width - 24 * Config(MyConfig.ScreenMultiplicator)) / (MaxGraphItems)
            GraphSpacing.Height = GraphDrawingSize.Height - 24 * Config(MyConfig.ScreenMultiplicator)
            GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.KiloMeters)

            Dim tempMax, tempAvr As Double
            Dim tempValue(1) As Double
            For J = MaxGraphItems To 1 Step -1
                Buffer.DrawLine(DimGrayDrawingPen, J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + 6 * Config(MyConfig.ScreenMultiplicator), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), GraphCount * GraphDrawingSize.Height + GraphDrawingSize.Height - 6 * Config(MyConfig.ScreenMultiplicator))

                If ShowGraph(0) = True Then
                    GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.KiloMeters)
                    Buffer.DrawLine(GraphDrawPen(0), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J).KiloMeters * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), (J - 1) * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J - 1).KiloMeters * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                    If ShowGraph(1) = False And ShowGraph(2) = False And ShowGraph(3) = False And ShowGraph(4) = False Then Buffer.DrawLine(GraphDrawPen(0), 12, (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.KiloMeters * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.KiloMeters * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                End If

                If ShowGraph(1) = True Then
                    GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.FuelAmount)
                    Buffer.DrawLine(GraphDrawPen(1), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J).FuelAmount * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), (J - 1) * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J - 1).FuelAmount * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                    If ShowGraph(0) = False And ShowGraph(2) = False And ShowGraph(3) = False And ShowGraph(4) = False Then Buffer.DrawLine(GraphDrawPen(1), 12, (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.FuelAmount * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.FuelAmount * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                End If

                If ShowGraph(2) = True Then
                    GraphFactor = GraphSpacing.Height / Math.Max(0.001, Stats.MinMax.Max.PricePerLiter)
                    Buffer.DrawLine(GraphDrawPen(2), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J).PricePerLiter * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), (J - 1) * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J - 1).PricePerLiter * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                    If ShowGraph(1) = False And ShowGraph(0) = False And ShowGraph(3) = False And ShowGraph(4) = False Then Buffer.DrawLine(GraphDrawPen(2), 12, (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.PricePerLiter * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.PricePerLiter * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                End If

                If ShowGraph(3) = True Then
                    GraphFactor = GraphSpacing.Height / Math.Max(0.01, Stats.MinMax.Max.TimeDiff)
                    Buffer.DrawLine(GraphDrawPen(3), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J).TimeDiff * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), (J - 1) * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - FuelList(J - 1).TimeDiff * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                    If ShowGraph(1) = False And ShowGraph(2) = False And ShowGraph(0) = False And ShowGraph(4) = False Then Buffer.DrawLine(GraphDrawPen(3), 12, (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.TimeDiff * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - Stats.Average.TimeDiff * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                End If

                If ShowGraph(4) = True Then
                    If Not Config(MyConfig.LastGraphType) = ConfigShow(MyConfig.LastGraphType, 1, ConfigType.Value) Then
                        tempMax = Stats.MinMax.Max.ConsumptionPer100
                        tempAvr = Stats.Average.ConsumptionPer100
                        tempValue(0) = FuelList(J - 1).ConsumptionPer100
                        tempValue(1) = FuelList(J).ConsumptionPer100
                    Else
                        tempMax = Stats.MinMax.Max.MilesPerGallon
                        tempAvr = Stats.Average.MilesPerGallon
                        tempValue(0) = FuelList(J - 1).MilesPerGallon
                        tempValue(1) = FuelList(J).MilesPerGallon
                    End If

                    GraphFactor = GraphSpacing.Height / Math.Max(0.01, tempMax)
                    Buffer.DrawLine(GraphDrawPen(4), J * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - tempValue(1) * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), (J - 1) * GraphSpacing.Width + 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - tempValue(0) * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                    If ShowGraph(1) = False And ShowGraph(2) = False And ShowGraph(3) = False And ShowGraph(0) = False Then Buffer.DrawLine(GraphDrawPen(4), 12, (GraphCount + 1) * GraphDrawingSize.Height - tempAvr * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator), GraphDrawingSize.Width - 12 * Config(MyConfig.ScreenMultiplicator), (GraphCount + 1) * GraphDrawingSize.Height - tempAvr * GraphFactor - 12 * Config(MyConfig.ScreenMultiplicator))
                End If
            Next
        End If
    End Sub

    Public Sub PreDrawAll()
        Call CreateOwnBackBuffer(MenuLoc.All, ObjectHeight(MenuLoc.All) * (ItemMax(MenuLoc.All) + 1))
        Min = OverViewPage * (OverViewItemCount)
        Max = Math.Min(FuelList.Length - 1, (OverViewPage + 1) * OverViewItemCount - 1)
        Dim PositionFix As Integer = -(Min - 1) * ObjectHeight(MenuLoc.All)
        If FuelList.Length > 0 Then
            Dim MyTempString As String
            If Min = 0 Then
                DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), 2 * BlackDrawingPen.Width * Config(MyConfig.ScreenMultiplicator), InnerDrawSize.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))
                'Get Size of the text to center it
                'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
                Buffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)
                Buffer.DrawString(ConstTempString, frmFuel.Font, BlackDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) / 2 - ConstTempSize.Height / 2)

                MyTempString = Stats.Average.KiloMeters & Config(MyConfig.MetricsUnit) & "(" & (Stats.All.DistanceState + Stats.Average.KiloMeters) & Config(MyConfig.MetricsUnit) & ")" & " - " & Stats.Average.LeftKiloMeters & Config(MyConfig.MetricsUnit) & " left" & vbNewLine & _
                             Stats.Average.FuelAmount & Config(MyConfig.FuelAmountUnit) & " - " & Stats.Average.LeftFuelAmount & Config(MyConfig.FuelAmountUnit) & " left" & vbNewLine & _
                             Stats.Average.PriceAmount & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                             Stats.Average.PricePerLiter & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                             Stats.Average.ConsumptionPer100 & Config(MyConfig.FuelAmountUnit) & vbNewLine & _
                             Stats.Average.MilesPerGallon & Config(MyConfig.MetricsUnit) & "/" & Config(MyConfig.FuelAmountUnit) & vbNewLine & _
                             Format(DateTime.Parse(FuelList(0).TimePrint).AddDays(Stats.Average.TimeDiff), "yyyy-MM-dd HH:mm") & " - " & Stats.Average.LeftTimeLength & " days" & " left"

                Buffer.DrawString(MyTempString, frmFuel.Font, BlackDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator) + ConstTempSize.Width, ObjectHeight(MenuLoc.All) / 2 - ConstTempSize.Height / 2)

                MyTempString = "Predict" '& FuelList(smallI).TimePrint
                tempSize = BackBuffer.MeasureString(MyTempString, frmFuel.Font)
                Buffer.DrawString(MyTempString, frmFuel.Font, FPSDrawingBrush, InnerDrawSize.Width - tempSize.Width - 6 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) / 2 - tempSize.Height / 2)
            Else
                DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), 2 * BlackDrawingPen.Width * Config(MyConfig.ScreenMultiplicator), InnerDrawSize.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))
                'Get Size of the text to center it
                'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
                Buffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)

                MyTempString = "<- Back (Page " & (OverViewPage + 1) & " of " & Math.Ceiling(FuelList.Length / OverViewItemCount) & ")"
                tempSize = BackBuffer.MeasureString(MyTempString, frmFuel.Font)
                Buffer.DrawString(MyTempString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, ObjectHeight(MenuLoc.All) / 2 - tempSize.Height / 2)
            End If

            For I = Min To Max
                DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), (PositionFix + I * ObjectHeight(MenuLoc.All)) + 2 * BlackDrawingPen.Width * Config(MyConfig.ScreenMultiplicator), InnerDrawSize.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))
                'Get Size of the text to center it
                'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
                Buffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)
                Buffer.DrawString(ConstTempString, frmFuel.Font, BlackDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator), (PositionFix + I * ObjectHeight(MenuLoc.All)) + ObjectHeight(MenuLoc.All) / 2 - ConstTempSize.Height / 2)

                MyTempString = FuelList(I).KiloMeters & Config(MyConfig.MetricsUnit) & "(" & FuelList(I).DistanceState & Config(MyConfig.MetricsUnit) & ")" & vbNewLine & _
                             FuelList(I).FuelAmount & Config(MyConfig.FuelAmountUnit) & vbNewLine & _
                             FuelList(I).PriceAmount & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                             FuelList(I).PricePerLiter & Config(MyConfig.CurrencyUnit) & vbNewLine & _
                             FuelList(I).ConsumptionPer100 & Config(MyConfig.FuelAmountUnit) & vbNewLine & _
                             FuelList(I).MilesPerGallon & Config(MyConfig.MetricsUnit) & "/" & Config(MyConfig.FuelAmountUnit) & vbNewLine & _
                             FuelList(I).TimePrint & " - " & FuelList(I).TimeDiff & " days"

                Buffer.DrawString(MyTempString, frmFuel.Font, BlackDrawingBrush, 6 * Config(MyConfig.ScreenMultiplicator) + ConstTempSize.Width, (PositionFix + I * ObjectHeight(MenuLoc.All)) + ObjectHeight(MenuLoc.All) / 2 - ConstTempSize.Height / 2)

                MyTempString = (I + 1) & ". " '& FuelList(smallI).TimePrint
                tempSize = BackBuffer.MeasureString(MyTempString, frmFuel.Font)
                Buffer.DrawString(MyTempString, frmFuel.Font, FPSDrawingBrush, InnerDrawSize.Width - tempSize.Width - 6 * Config(MyConfig.ScreenMultiplicator), (PositionFix + I * ObjectHeight(MenuLoc.All)) + ObjectHeight(MenuLoc.All) / 2 - tempSize.Height / 2)
            Next
            If GetOverViewFix() = 0 Then
                DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), 2 * BlackDrawingPen.Width * Config(MyConfig.ScreenMultiplicator), InnerDrawSize.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.All) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))
                'Get Size of the text to center it
                'If I = SelectedItem Then BackBuffer.FillRectangle(SkyBlueDrawingBrush, DrawingObjectRectangle)
                Buffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)

                MyTempString = "(Page " & (OverViewPage + 1) & " of " & Math.Ceiling(FuelList.Length / OverViewItemCount) & ") Next ->"
                tempSize = BackBuffer.MeasureString(MyTempString, frmFuel.Font)
                Buffer.DrawString(MyTempString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, (OverViewItemCount + 1) * ObjectHeight(MenuLoc.All) + ObjectHeight(MenuLoc.All) / 2 - tempSize.Height / 2)
            End If
        End If
    End Sub

    Public Sub PreDrawAdd(Optional ByVal SpareDraw As Boolean = False)
        Call CreateOwnBackBuffer(MenuLoc.Add, ObjectHeight(MenuLoc.Add) * (ItemMax(MenuLoc.Add) + 1))

        Dim AddHeaders() As String = {"Distance", "Fuel", "Price p. " & Config(MyConfig.FuelAmountUnit)}
        Dim NumberUnit() As String = {Config(MyConfig.MetricsUnit), Config(MyConfig.FuelAmountUnit), Config(MyConfig.CurrencyUnit)}
        Dim tempPoints(2) As Point

        For I = 0 To AddHeaders.Length - 1
            DrawingObjectRectangle = New Rectangle(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Add) + BlackDrawingPen.Width, InnerDrawSize.Width - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator), ObjectHeight(MenuLoc.Add) - BlackDrawingPen.Width * 4 * Config(MyConfig.ScreenMultiplicator))

            Buffer.DrawRectangle(BlackDrawingPen, DrawingObjectRectangle)
            tempSize = Buffer.MeasureString(AddHeaders(I), frmFuel.Font)
            Buffer.DrawString(AddHeaders(I), frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight / 2)
            For J = NumberSpan(I, 0) To NumberSpan(I, 1)
                Buffer.FillRectangle(BlackDrawingBrush, BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 - NumberFieldSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 3, NumberFieldSize.Width, NumberFieldSize.Height)
                Buffer.DrawString(AddNumbers(I, J), frmFuel.Font, WhiteDrawingBrush, BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 - NumberSize(AddNumbers(I, J)).Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 3.5 + NumberFieldSize.Height / 2 - NumberSize(AddNumbers(I, J)).Height)

                'Top Arrow
                tempPoints(0) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 - NumberFieldSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 2.5)
                tempPoints(1) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 + NumberFieldSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 2.5)
                tempPoints(2) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 1.5)
                Buffer.FillPolygon(BlackDrawingBrush, tempPoints)
                'Bottom Arrow
                tempPoints(0) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 - NumberFieldSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + NumberFieldSize.Height + FontLineHeight * 3.5)
                tempPoints(1) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2 + NumberFieldSize.Width / 2, I * ObjectHeight(MenuLoc.Add) + NumberFieldSize.Height + FontLineHeight * 3.5)
                tempPoints(2) = New Point(BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + J * ArrowCountPerLine + ArrowCountPerLine / 2, I * ObjectHeight(MenuLoc.Add) + NumberFieldSize.Height + FontLineHeight * 4.5)
                Buffer.FillPolygon(BlackDrawingBrush, tempPoints)
            Next
            If Not Config(MyConfig.DistanceAddType) = "total" Or I > 0 Then Buffer.DrawString(",", frmFuel.Font, BlackDrawingBrush, BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + 3 * ArrowCountPerLine + ArrowCountPerLine - Buffer.MeasureString(",", frmFuel.Font).Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 3.5 + NumberFieldSize.Height / 2 - Buffer.MeasureString(",", frmFuel.Font).Height)
            Buffer.DrawString(NumberUnit(I), frmFuel.Font, BlackDrawingBrush, BlackDrawingPen.Width * 2 * Config(MyConfig.ScreenMultiplicator) + 7 * ArrowCountPerLine + ArrowCountPerLine / 2 - Buffer.MeasureString(NumberUnit(I), frmFuel.Font).Width / 2, I * ObjectHeight(MenuLoc.Add) + FontLineHeight * 3.5 + NumberFieldSize.Height / 2 - Buffer.MeasureString(NumberUnit(I), frmFuel.Font).Height)
        Next
        Dim SaveString As String = "Save"
        tempSize = Buffer.MeasureString(SaveString, frmFuel.Font)
        Buffer.DrawString(SaveString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, 3 * ObjectHeight(MenuLoc.Add) + ObjectHeight(MenuLoc.Add) / 2 - tempSize.Height / 2)
        Buffer.DrawRectangle(BlackDrawingPen, InnerDrawSize.Width / 4, 3 * ObjectHeight(MenuLoc.Add) + ObjectHeight(MenuLoc.Add) / 4, InnerDrawSize.Width / 2, ObjectHeight(MenuLoc.Add) / 2)

        Dim DateString As String = Format(AddTimePrint, "yyyy-MM-dd HH:mm")
        tempSize = Buffer.MeasureString(DateString, frmFuel.Font)
        Buffer.DrawString(DateString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, 4 * ObjectHeight(MenuLoc.Add) + ObjectHeight(MenuLoc.Add) / 2 - tempSize.Height / 2)
        Buffer.DrawRectangle(BlackDrawingPen, InnerDrawSize.Width / 4, 4 * ObjectHeight(MenuLoc.Add) + ObjectHeight(MenuLoc.Add) / 4, InnerDrawSize.Width / 2, ObjectHeight(MenuLoc.Add) / 2)
    End Sub

    Public Sub PreDrawConfig()
        Dim I As Integer
        Dim tempText As String
        Dim tempPoints(2) As Point

        Call CreateOwnBackBuffer(MenuLoc.Config, ObjectHeight(MenuLoc.Config) * (ItemMax(MenuLoc.Config) + 1))
        For I = 0 To MyConfig.EndConfig - 1
            tempSize = ConfigNameSize(I)
            Buffer.DrawString(ConfigName(I), frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, ObjectHeight(MenuLoc.Config) * I + FontLineHeight * 0.5)

            If Not ConfigShow(I, ConfigSelected(I), ConfigType.Value) = "custom" Then
                tempSize = ConfigShowSize(I, ConfigSelected(I), ConfigType.Text)
                Buffer.DrawString(ConfigShow(I, ConfigSelected(I), ConfigType.Text), frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, ObjectHeight(MenuLoc.Config) * I + FontLineHeight * 3)
            Else
                tempText = ConfigShow(I, ConfigSelected(I), ConfigType.Text) & ": " & ConfigCustom(I)
                tempSize = Buffer.MeasureString(tempText, frmFuel.Font)
                Buffer.DrawString(tempText, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, ObjectHeight(MenuLoc.Config) * I + FontLineHeight * 3)
            End If

            Buffer.DrawRectangle(BlackDrawingPen, 48 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 2, InnerDrawSize.Width - 48 * 2 * Config(MyConfig.ScreenMultiplicator), FontLineHeight * 3)
            Buffer.DrawLine(BlackDrawingPen, 0, (I + 1) * ObjectHeight(MenuLoc.Config), InnerDrawSize.Width, (I + 1) * ObjectHeight(MenuLoc.Config))

            'Left Arrow
            tempPoints(0) = New Point(5 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 3.5)
            tempPoints(1) = New Point((48 - 5) * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 2)
            tempPoints(2) = New Point((48 - 5) * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 5)
            Buffer.FillPolygon(BlackDrawingBrush, tempPoints)
            'Right Arrow
            tempPoints(0) = New Point(InnerDrawSize.Width - 5 * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 3.5)
            tempPoints(1) = New Point(InnerDrawSize.Width - (48 - 5) * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 2)
            tempPoints(2) = New Point(InnerDrawSize.Width - (48 - 5) * Config(MyConfig.ScreenMultiplicator), I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 5)
            Buffer.FillPolygon(BlackDrawingBrush, tempPoints)
        Next
        I = MyConfig.EndConfig
        Dim SaveString As String = "Save"
        tempSize = Buffer.MeasureString(SaveString, frmFuel.Font)
        Buffer.DrawString(SaveString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, I * ObjectHeight(MenuLoc.Config) + ObjectHeight(MenuLoc.Config) / 2 - tempSize.Height / 2)
        Buffer.DrawRectangle(BlackDrawingPen, InnerDrawSize.Width / 4, I * ObjectHeight(MenuLoc.Config) + ObjectHeight(MenuLoc.Config) / 4, InnerDrawSize.Width / 2, ObjectHeight(MenuLoc.Config) / 2)

        Buffer.DrawLine(BlackDrawingPen, 0, (I + 1) * ObjectHeight(MenuLoc.Config), InnerDrawSize.Width, (I + 1) * ObjectHeight(MenuLoc.Config))

        I += 1
        Dim ImExString As String = "CSV Export/Import(Fuel Data)"
        tempSize = Buffer.MeasureString(ImExString, frmFuel.Font)
        Buffer.DrawString(ImExString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 2 - tempSize.Width / 2, I * ObjectHeight(MenuLoc.Config) + FontLineHeight * 0.5)

        ImExString = "Export"
        tempSize = Buffer.MeasureString(ImExString, frmFuel.Font)
        Buffer.DrawString(ImExString, frmFuel.Font, BlackDrawingBrush, InnerDrawSize.Width / 4 - tempSize.Width / 2, I * ObjectHeight(MenuLoc.Config) + (ObjectHeight(MenuLoc.Config) - FontLineHeight * 1.5) / 2 - tempSize.Height / 2 + FontLineHeight * 1.5)

        ImExString = "Import"
        tempSize = Buffer.MeasureString(ImExString, frmFuel.Font)
        Buffer.DrawString(ImExString, frmFuel.Font, BlackDrawingBrush, 3 * InnerDrawSize.Width / 4 - tempSize.Width / 2, I * ObjectHeight(MenuLoc.Config) + (ObjectHeight(MenuLoc.Config) - FontLineHeight * 1.5) / 2 - tempSize.Height / 2 + FontLineHeight * 1.5)

        Buffer.DrawRectangle(BlackDrawingPen, InnerDrawSize.Width / 8, I * ObjectHeight(MenuLoc.Config) + (ObjectHeight(MenuLoc.Config) - FontLineHeight * 1.5) / 4 + FontLineHeight * 1.5, InnerDrawSize.Width / 4, FontLineHeight * 2)
        Buffer.DrawRectangle(BlackDrawingPen, 5 * InnerDrawSize.Width / 8, I * ObjectHeight(MenuLoc.Config) + (ObjectHeight(MenuLoc.Config) - FontLineHeight * 1.5) / 4 + FontLineHeight * 1.5, InnerDrawSize.Width / 4, FontLineHeight * 2)

    End Sub

    Public Sub PreDrawAbout()
        Call CreateOwnBackBuffer(MenuLoc.About, InnerDrawSize.Height)
    End Sub

    Public Sub PreDrawEndMenu()
        Call CreateOwnBackBuffer(MenuLoc.EndMenu, InnerDrawSize.Height)
        Dim ExitString As String = "Exit"
        Dim ExitSize As SizeF = Buffer.MeasureString(ExitString, frmFuel.Font)
        Dim NewFont As New Font(frmFuel.Font.Name, Math.Min((frmFuel.Font.Size / ExitSize.Width) * InnerDrawSize.Width, (frmFuel.Font.Size / ExitSize.Height) * InnerDrawSize.Height) * 0.9, FontStyle.Regular)
        ExitSize = Buffer.MeasureString(ExitString, NewFont)
        Buffer.DrawString(ExitString, NewFont, BlackDrawingBrush, (InnerDrawSize.Width / 2) - (ExitSize.Width / 2), (InnerDrawSize.Height / 2) - (ExitSize.Height / 2))
    End Sub

    Public Sub DrawAboutPause()
        If tmrDirectX.Enabled = False Then
            Buffer = Graphics.FromImage(BufferBitMap(MenuLoc.About))
            Buffer.FillEllipse(New SolidBrush(Color.Red), InnerDrawSize.Width - (Config(MyConfig.ScreenMultiplicator) * 2 + Config(MyConfig.ScreenMultiplicator) * 16), Config(MyConfig.ScreenMultiplicator) * 2, Config(MyConfig.ScreenMultiplicator) * 16, Config(MyConfig.ScreenMultiplicator) * 16)
        End If
    End Sub

    Public Sub DrawToSurface()
        Call DrawMenu()
        CreateBackBufferDraw()
        BackBufferDraw.DrawImage(BufferBitMap(CurrentLocation), 0, CurrentPosition(CurrentLocation).Y)
        If ObjectHeight(CurrentLocation) * (ItemMax(CurrentLocation) + 1) > InnerDrawSize.Height Then
            BackBufferDraw.FillRectangle(New SolidBrush(Color.Gray), frmFuel.pctDraw.Width - 4 * Config(MyConfig.ScreenMultiplicator), (-CurrentPosition(CurrentLocation).Y) / (ObjectHeight(CurrentLocation) * (ItemMax(CurrentLocation) + 1)) * InnerDrawSize.Height, 4 * Config(MyConfig.ScreenMultiplicator), InnerDrawSize.Height * InnerDrawSize.Height / (ObjectHeight(CurrentLocation) * (ItemMax(CurrentLocation) + 1)))
        End If
        BackBufferDraw.DrawLine(BlackDrawingPen, 0, InnerDrawSize.Height, frmFuel.pctDraw.Width, InnerDrawSize.Height)
        BackBufferDraw.DrawImage(BackBufferMenuBitMap, 0, InnerDrawSize.Height + 1)
        frmFuel.pctDraw.Image = BackBufferDrawBitMap
        NeedRefresh = True
    End Sub

    Public Sub DrawMenuScroll()
        Call DrawMenu()
        BackBufferDraw.FillRectangle(WhiteDrawingBrush, 0, 0, frmFuel.pctDraw.Width, InnerDrawSize.Height)
        BackBufferDraw.DrawImage(MenuIconBitMap(CurrentLocation), (frmFuel.pctDraw.Width / 2) - (IconSize.Width / 2), (InnerDrawSize.Height / 2) - (IconSize.Height / 2) - (FontLineHeight / 4))
        BackBufferDraw.DrawString(IconText(CurrentLocation), frmFuel.Font, BlackDrawingBrush, (frmFuel.pctDraw.Width / 2) - (IconTextSize(CurrentLocation).Width / 2), (InnerDrawSize.Height / 2) + (FontLineHeight / 4) + (IconSize.Height / 2))
        BackBufferDraw.DrawImage(BackBufferMenuBitMap, 0, InnerDrawSize.Height + 1)
        BackBufferDraw.DrawLine(BlackDrawingPen, 0, InnerDrawSize.Height, frmFuel.pctDraw.Width, InnerDrawSize.Height)
        frmFuel.pctDraw.Image = BackBufferDrawBitMap
        NeedRefresh = True
    End Sub

End Module
